/* ======================================================
	External libraries
====================================================== */

/*
	Yet another thread library for OpenSCAD by arpruss is licensed under the Creative Commons - Attribution license. 
	https://www.thingiverse.com/thing:2277141
	https://www.thingiverse.com/arpruss
	
	No changes made, except commenting out the example thread to use the library.
*/
include <lib/quickthread.scad>;

/* ======================================================
	Config
====================================================== */

container_height 	= 90;
container_width 	= 60;
container_wall 		= 3;
container_border	= 10;
container_opening = 43;

thread_height			=	10;
thread_pitch 			= 2;
thread_angle 			= 5;
thread_lead				= 5;

fill_indicators		= "yes";

/* ======================================================
	Code
====================================================== */

height_60 = container_height / 100 * 60;
height_80 = container_height / 100 * 80;

// container
difference () {
	cylinder(h=container_height, d=container_width, $fn=128);
	translate([0,0,container_wall])
	cylinder(h=container_height-container_wall, d=container_width-container_wall, $fn=128);
	
	// fill markings
	if (fill_indicators == "yes") {
		translate([0,0,height_60]) ring();
		translate([0,0,height_80]) ring();
	}
}

// opening
translate([0,0,container_height])
difference () {
	cylinder(h=container_border, d=container_width, $fn=128);
	cylinder(h=container_border, d1=container_width-container_wall, d2=container_opening-container_wall*2, $fn=128);
}

// thread
translate([0,0,container_height+thread_height])
difference () {
	isoThread(d=container_opening+container_wall,h=thread_height,pitch=thread_pitch,angle=thread_angle,internal=false, lead=thread_lead , $fn=64);
	cylinder(d=container_opening-container_wall*2, h=thread_height+container_wall, $fn=128);
}

module ring () {
	difference () {
		$fn=128;
		cylinder(d=container_width+container_wall, h=1);
		cylinder(d=container_width-0.2, h=1);
	}
}